<cfcomponent displayname="Form Rule" extends="farcry.farcry_core.packages.rules.rules" hint="Rule for Creating Forms" bCustomType="1">

<!--- rule object properties --->
<cfproperty name="title" type="string" hint="Title for form" required="no" default="">
<cfproperty name="type" type="string" hint="The type that the form is based on" required="yes" default="">
<cfproperty name="formfields" type="longchar" hint="WDDX encoded query containing the form fields" required="yes" default="">
<cfproperty name="successmessage" type="string" hint="The message displayed when the form is successfully submitted" required="yes" default="Data successfully submitted">
<cfproperty name="processing" type="longchar" hint="A skin that contains processing code" required="no" default="">

<!--- pseudo import tag library --->
<cfimport prefix="q4" taglib="/farcry/fourq/tags">
<cfimport prefix="nj" taglib="/farcry/farcry_core/tags/navajo">
<cfimport taglib="/farcry/farcry_core/tags/widgets" prefix="widgets">

<cffunction name="isCreditCard" hint="Checks the validity of a credit card number" access="private" output="false">
	<cfargument name="ccNo" type="string" required="true">
	<cfargument name="cardType" type="string" required="false">
	
	<!---
	 * Returns TRUE if the string is a valid credit card number.
	 * Modded by RCamden - Check for any non numeric and return false.
	 * Modded by Author - fixed mastercard checking
	 * Updated to use [:digit:] and allow spaces
	 * Corrected nondigit check
	 * 
	 * @param ccNo 	 The credit card number. (Required)
	 * @param cardType 	 One of: AMEX, DINERS, DISCOVER, MASTERCARD, VISA (Optional)
	 * @return Returns a boolean. 
	 * @author Nick de Voil (nick@devoil.com) 
	 * @version 4, August 17, 2004 
	--->
	
	<cfscript>
		var rv = "";
		var str = "";
		var chk = 0;
		var ccln = 0;
		var strln = 0;
		var i = 1;
	
		if(reFind("[^[:digit:]]",ccNo)) return FALSE;
	        ccNo = replace(ccNo," ","","ALL");
		rv = Reverse(ccNo);
		ccln = Len(ccNo);
		if(ccln lt 12) return FALSE;
		for(i = 1; i lte ccln;  i = i + 1) {
			if(i mod 2 eq 0) {
				str = str & Mid(rv, i, 1) * 2;
			} else {
				str = str & Mid(rv, i, 1);
			}
		}
		strln = Len(str);
		for(i = 1; i lte strln; i = i + 1) chk = chk + Mid(str, i, 1);
		if((chk neq 0) and (chk mod 10 eq 0)) {
			if(ArrayLen(Arguments) lt 2) return TRUE;
			switch(UCase(Arguments[2])) {
			case "AMEX":		if ((ccln eq 15) and (((Left(ccNo, 2) is "34")) or ((Left(ccNo, 2) is "37")))) return TRUE; break;
			case "DINERS":		if ((ccln eq 14) and (((Left(ccNo, 3) gte 300) and (Left(ccNo, 3) lte 305)) or (Left(ccNo, 2) is "36") or (Left(ccNo, 2) is "38"))) return TRUE; break;
			case "DISCOVER":	if ((ccln eq 16) and (Left(ccNo, 4) is "6011")) return TRUE; break;
			case "MASTERCARD":	if ((ccln eq 16) and (Left(ccNo, 2) gte 51) and (Left(ccNo, 2) lte 55)) return TRUE; break;
			case "VISA":		if (((ccln eq 13) or (ccln eq 16)) and (Left(ccNo, 1) is "4")) return TRUE; break;
			default: return TRUE;
			}
		}
		return FALSE;
	</cfscript>
</cffunction>
		
<cffunction name="forminput" output="false" returntype="string">
	<cfargument name="index" required="yes" type="string">
	<cfargument name="type" required="yes" type="string">
	<cfargument name="params" required="no" type="string" default=''>
	<cfargument name="default" required="no" type="string" default=''>

	<cfsavecontent variable="returnval">
		<cfoutput>
			<cfswitch expression="#type#">
				<cfcase value="email">
					<input type='text' name='field_#arguments.index#' id='field_#arguments.index#' value='#arguments.default#' maxlength='150'>
				</cfcase>
				<cfcase value="text">
					<input type='text' name='field_#arguments.index#' id='field_#arguments.index#' value='#arguments.default#' maxlength='#arguments.params#'>
				</cfcase>
				<cfcase value="regex">
					<input type='text' name='field_#arguments.index#' id='field_#arguments.index#' value='#arguments.default#' maxlength='255'>
				</cfcase>
				<cfcase value="selection">
					<select name='field_#arguments.index#' id='field_#arguments.index#'>
					<cfloop list="#arguments.params#" index="thisoption">
						<option <cfif thisoption is arguments.default>selected</cfif>>#thisoption#</option>
					</cfloop>
					</select>
				</cfcase>
				<cfcase value="postcode">
					<input type='text' name='field_#arguments.index#' id='field_#arguments.index#' value='#arguments.default#' maxlength='5'>
				</cfcase>
				<cfcase value="phonenumber">
					<input type='text' name='field_#arguments.index#' id='field_#arguments.index#' value='#arguments.default#' maxlength='20'>
				</cfcase>
				<cfcase value="creditcard">
					<input type='text' name='field_#arguments.index#' id='field_#arguments.index#' value='#arguments.default#' maxlength='19'>
				</cfcase>
				<cfcase value="textarea">
					<textarea name='field_#arguments.index#' id='field_#arguments.index#' maxlength='2000' cols=60 rows=4>#arguments.default#</textarea>
				</cfcase>
				<cfcase value="pastdate">
					<cfif not isdefined("arguments.default") or not len(trim(arguments.default))><cfset arguments.default="#lsdateformat(now(),'long')#"></cfif>
					<select name='day_#arguments.index#' value='#day(lsparsedatetime(arguments.default))#' size='0' id='day_#arguments.index#' onBlur="$('field_#arguments.index#').value=$('day_#arguments.index#').value + ' ' + $('month_#arguments.index#').value + ' ' + $('year_#arguments.index#').value;">
					<cfloop from="1" to="31" index="i">
						<option value='#i#'<cfif day(lsparsedatetime(arguments.default)) is i> selected</cfif>>#i#</option>
					</cfloop>
				 	</select>
					<select name='month_#arguments.index#' value='#month(lsparsedatetime(arguments.default))#' size='0' id='month_#arguments.index#' onBlur="$('field_#arguments.index#').value=$('day_#arguments.index#').value + ' ' + $('month_#arguments.index#').value + ' ' + $('year_#arguments.index#').value;">
					<cfloop from="1" to="12" index="i">
						<option value='#i#'<cfif day(lsparsedatetime(arguments.default)) is i> selected</cfif>>#monthasstring(i)#</option>
					</cfloop>
				 	</select>
					<select name='year_#arguments.index#' value='#month(lsparsedatetime(arguments.default))#' size='0' id='year_#arguments.index#' onBlur="$('field_#arguments.index#').value=$('day_#arguments.index#').value + ' ' + $('month_#arguments.index#').value + ' ' + $('year_#arguments.index#').value;">
					<cfloop from="#year(now())#" to="#year(now()) - 85#" index="i" step='-1'>
						<option value='#i#'<cfif day(lsparsedatetime(arguments.default)) is i >selected</cfif>>#i#</option>
					</cfloop>
				 	</select>
					<input type='hidden' name='field_#arguments.index#' id='question_#arguments.index#' value='#arguments.default#'>
				</cfcase>
				<cfcase value="futuredate">
					<cfif not isdefined("arguments.default") or not len(trim(arguments.default))><cfset arguments.default="#lsdateformat(now(),'long')#"></cfif>
					<select name='day_#arguments.index#' value='#day(lsparsedatetime(arguments.default))#' size='0' id='day_#arguments.index#' onBlur="$('field_#arguments.index#').value=$('day_#arguments.index#').value + ' ' + $('month_#arguments.index#').value + ' ' + $('year_#arguments.index#').value;">
					<cfloop from="1" to="31" index="i">
						<option value='#i#'<cfif day(lsparsedatetime(arguments.default)) is i> selected</cfif>>#i#</option>
					</cfloop>
				 	</select>
					<select name='month_#arguments.index#' value='#month(lsparsedatetime(arguments.default))#' size='0' id='month_#arguments.index#' onBlur="$('field_#arguments.index#').value=$('day_#arguments.index#').value + ' ' + $('month_#arguments.index#').value + ' ' + $('year_#arguments.index#').value;">
					<cfloop from="1" to="12" index="i">
						<option value='#i#'<cfif month(lsparsedatetime(arguments.default)) is i> selected</cfif>>#monthasstring(i)#</option>
					</cfloop>
				 	</select>
					<select name='year_#arguments.index#' value='#month(lsparsedatetime(arguments.default))#' size='0' id='year_#arguments.index#' onBlur="$('field_#arguments.index#').value=$('day_#arguments.index#').value + ' ' + $('month_#arguments.index#').value + ' ' + $('year_#arguments.index#').value;">
					<cfloop from="#year(now())#" to="#year(now()) + 85#" index="i" step='1'>
						<option value='#i#'<cfif year(lsparsedatetime(arguments.default)) is i> selected</cfif>>#i#</option>
					</cfloop>
				 	</select>
					<input type='hidden' name='field_#arguments.index#' id='question_#arguments.index#' value='#arguments.default#'>
				</cfcase>
				<cfcase value="time">
					
				</cfcase>
				<cfcase value="manualdate">
					<cfif not isdefined("arguments.default") or not len(trim(arguments.default))><cfset arguments.default="#lsdateformat(now(),'long')#"></cfif>
					<input type='text' name='field_#arguments.index#' id='field_#arguments.index#' value='#lsdateformat(arguments.default,"long")#' maxlength='50'">
				</cfcase>
				<cfcase value="decimal">
					<input type='text' name='field_#arguments.index#' id='field_#arguments.index#' value='#arguments.default#' maxlength='50'>
				</cfcase>
				<cfcase value="integer">
					<input type='text' name='field_#arguments.index#' id='field_#arguments.index#' value='#arguments.default#' maxlength='50'>
				</cfcase>
				<cfcase value="money">
					<input type='text' name='field_#arguments.index#' id='field_#arguments.index#' value='#arguments.default#' maxlength='10'>
				</cfcase>
				<cfcase value="truefalse">
					<select name='field_#arguments.index#' id='field_#arguments.index#'>
						<option value='true' <cfif arguments.default is "true">selected</cfif>>True</option>
						<option value='false' <cfif arguments.default is "false">selected</cfif>>False</option>
					</select>
				</cfcase>
				<cfcase value="yesno">
					<select name='field_#arguments.index#' id='field_#arguments.index#'>
						<option value='true' <cfif arguments.default is "true">selected</cfif>>Yes</option>
						<option value='false' <cfif arguments.default is "false">selected</cfif>>No</option>
					</select>
				</cfcase>
				<cfcase value="checkbox">
					<input type='checkbox' name='field_#arguments.index#' id='field_#arguments.index#' <cfif arguments.default is "true">checked></cfif> value='true'>
				</cfcase>
				<cfcase value="ignore">
					<!--- There is to be no input for this field --->
				</cfcase>
			</cfswitch>
		</cfoutput>
	</cfsavecontent>

	<cfreturn returnval>
</cffunction>

<cffunction name="validateField" output="false" returntype="string">
	<cfargument name="whichfield" required="true" type="numeric">
	<cfargument name="required" required="true" type="numeric">
	<cfargument name="type" required="true" type="string">
	<cfargument name="caption" required="true" type="string">
	<cfargument name="parameters" required="true" type="string">

	<cfsavecontent variable="returnval">
		<cfoutput>
			<cfif arguments.type neq "checkbox" and arguments.required and not (structkeyexists(form,"field_#whichfield#") and len(trim(form["field_#whichfield#"])))>
				<li>#arguments.caption# is required. Please enter a value.</li>
			<cfelseif structkeyexists(form,"field_#whichfield#") and len(trim(form["field_#whichfield#"]))>
				<cfset form["field_#whichfield#"]=trim(form["field_#whichfield#"])>
				<cfswitch expression="#arguments.type#">
					<cfcase value="email">
						<cfif REFind("[^\w\@\.]",form["field_#whichfield#"]) or not REFind("^[A-Za-z][\w]*(\.[A-Za-z][\w]*)?\@[A-Za-z][\w]*(\.[A-Za-z][\w]{1,3}){1,3}(\.[A-Za-z]{1,2})?$",form["field_#whichfield#"])><li>#arguments.caption# is not a valid email</li></cfif>
					</cfcase>
					<cfcase value="text">
						<cfif REFind("[^\w;:\\/*!$%&()?.,+-@=_\s]",form["field_#whichfield#"])><li>#arguments.caption# contains invalid characters. Valid characters are alphanumeric, ;, :, \, /, *, !, $, %, &, (, ), ?, ., +, -, @, =, _, and ','</li></cfif>
						<cfif len(form["field_#whichfield#"]) gt val(arguments.parameters)><li>#arguments.caption# is too long - the maximum length for this field is #arguments.parameters#</li></cfif>
					</cfcase>
					<cfcase value="regex">
						<cfif not REFind("#arguments.parameters#",form["field_#whichfield#"])><li>#arguments.caption# does not match the required pattern.</li></cfif>
					</cfcase>
					<cfcase value="selection">
						<!--- There is no validation needed for this input type --->
					</cfcase>
					<cfcase value="postcode">
						<cfif not REFind("^\d{4,5}$",form["field_#whichfield#"])><li>#arguments.caption# is not a valid postcode</li></cfif>
					</cfcase>
					<cfcase value="phonenumber">
						<cfif not REFind("^(\(\d{1,5}\))?[ \d]{8,15}$",form["field_#whichfield#"])><li>#arguments.caption# is not a valid phone number</li></cfif>
					</cfcase>
					<cfcase value="creditcard">
						<cfif iscreditcard(ccNo=form["field_#whichfield#"])><li>#arguments.caption# is not a valid credit card number</li></cfif>
					</cfcase>
					<cfcase value="textarea">
						<cfif REFind("[^\w;:\\/*!$%&()?.,+-@=_\s]",form["field_#whichfield#"])><li>#arguments.caption# contains invalid characters. Valid characters are alphanumeric, ;, :, \, /, *, !, $, %, &, (, ), ?, ., +, -, @, =, _, and ','</li></cfif>
						<cfif len(form["field_#whichfield#"]) gt 2000><li>#arguments.caption# is too long - the maximum length for this field is 2000 characters</li></cfif>
					</cfcase>
					<cfcase value="pastdate,futuredate,time,manualdate">
						<cfif not isdate(form["field_#whichfield#"])><li>#arguments.caption# is not a valid date</li></cfif>
					</cfcase>
					<cfcase value="decimal">
						<cfif not isnumeric(form["field_#whichfield#"])><li>#arguments.caption# is not a valid number</li></cfif>
					</cfcase>
					<cfcase value="integer">
						<cfif REFind("[^\d]",form["field_#whichfield#"])><li>#arguments.caption# must a whole number</li></cfif>
					</cfcase>
					<cfcase value="money">
						<cfif not lsiscurrency(form["field_#whichfield#"])><li>#arguments.caption# is not a valid dollar amount</li></cfif>
					</cfcase>
					<cfcase value="truefalse">
						<!--- There is no validation needed for this input type --->
					</cfcase>
					<cfcase value="yesno">
						<!--- There is no validation needed for this input type --->
					</cfcase>
					<cfcase value="checkbox">
						<!--- There is no validation needed for this input type --->
					</cfcase>
					<cfcase value="ignore">
						<!--- There is no validation needed for this input type --->
					</cfcase>
				</cfswitch>
			</cfif>
		</cfoutput>
	</cfsavecontent>

	<cfreturn returnval>
</cffunction>

<cffunction name="createPropertyQuery" returntype="query" output="false">
	<cfargument name="type" required="yes" type="string">
	
	<cfscript>
		var qFormFields=querynew("Property,Caption,Display,Default,Required,Type,Parameters,Separated,Position","varchar,varchar,bit,varchar,integer,varchar,varchar,bit,integer");
 
		for (thisprop in application.types[form.type].stprops)
		{
			if (not listcontains("locked,lastupdatedby,datetimecreated,ownedby,ObjectID,createdby,label,lockedBy,datetimelastupdated", trim(thisprop)))
			{
				sttmp = application.types[form.type].stprops[thisprop];
				
				queryaddrow(qFormFields);
				querysetcell(qFormFields,"Property",thisprop);
				querysetcell(qFormFields,"Caption",thisprop);
				querysetcell(qFormFields,"Display",1);
				querysetcell(qFormFields,"Position",qFormFields.recordcount);
				if (isdefined("sttmp.metadata.default"))
					querysetcell(qFormFields,"Default",sttmp.metadata.default);
				else
					querysetcell(qFormFields,"Default","");
				querysetcell(qFormFields,"Separated","0");
				querysetcell(qFormFields,"Required",iif(isdefined("sttmp.metadata.required") and sttmp.metadata.required,"2","0"));
				switch (sttmp.metadata.type)
				{
					case "nstring": case "string":
						querysetcell(qFormFields,"Type","text");
						if (find(thisprop,"phone")) querysetcell(qFormFields,"Type","phonenumber");
						if (find(thisprop,"email")) querysetcell(qFormFields,"Type","email");
						if (find(thisprop,"creditcard")) querysetcell(qFormFields,"Type","creditcard");
						if (find(thisprop,"postcode")) querysetcell(qFormFields,"Type","postcode");
						if (find(thisprop,"amount")) querysetcell(qFormFields,"Type","money");
						querysetcell(qFormFields,"Parameters","250");
						break;
					case "longchar":
						querysetcell(qFormFields,"Type","longchar");
						querysetcell(qFormFields,"Parameters","");
						break;
					case "date":
						querysetcell(qFormFields,"Type","pastdate");
						querysetcell(qFormFields,"Parameters","");
						break;
					case "numeric":
						querysetcell(qFormFields,"Type","integer");
						if (find(thisprop,"amount")) querysetcell(qFormFields,"Type","money");
						querysetcell(qFormFields,"Parameters","");
						break;
					case "boolean":
						querysetcell(qFormFields,"Type","checkbox");
						querysetcell(qFormFields,"Parameters","");
						break;
					case "array": case "uuid":
						querysetcell(qFormFields,"Type","ignore");
						break;
				}
			}
		}
		
		return qFormFields;
	</cfscript>
</cffunction>

<cffunction name="update" output="true">
	<cfargument name="objectID" required="Yes" type="uuid" default="">
	<cfargument name="label" required="no" type="string" default="">
	
	<cfset var stObj = this.getData(arguments.objectID)>
	<cfset var qFormFields=querynew("Property,Caption,Display,Default,Required,Type,Parameters,Separated,Position","varchar,varchar,bit,varchar,integer,varchar,varchar,bit,integer")>

	<cfif isdefined("url.submit")><cfset form.submit=url.submit></cfif>
	<cfparam name="form.title" default="#stObj.title#">
	<cfparam name="form.type" default="#stObj.type#">
	<cfparam name="form.processing" default="#stObj.processing#">
	<cfparam name="form.successmessage" default="#stObj.successmessage#">
	<cfparam name="form.formfields" default="#stObj.formfields#">
	
	<cfif isdefined("form.submit") and form.submit is "Go">
		<cfif form.type neq ""><cfset qFormFields=createPropertyQuery(form.type)></cfif>
		
		<cfscript>
			if (form.type eq stObj.type)
			{
				for (i=1;i lte qFormFields.recordcount; i=i+1)
				{
					if (isdefined("form.caption_#i#"))
					{
						querysetcell(qFormFields,"Caption",HTMLEditFormat(form["caption_#i#"]),i);
						querysetcell(qFormFields,"Display",iif(isdefined("form.display_#i#"),1,0),i);
						querysetcell(qFormFields,"Default",form["default_#i#"],i);
						querysetcell(qFormFields,"Separated",iif(isdefined("form.separated_#i#"),1,0),i);
						querysetcell(qFormFields,"Position",form["position_#i#"],i);
						if (isdefined("form.required_#i#"))
							querysetcell(qFormFields,"Required",form["required_#i#"],i);
						else
							querysetcell(qFormFields,"Required",0,i);
						if (form["type_" & i] neq type)
						{
							switch (form['type_' & i])
							{
								case "text":
									querysetcell(qFormFields,"Parameters","255",i);
									break;
								case "selection":
									querysetcell(qFormFields,"Parameters",HTMLEditFormat("Option 1,Option 2"),i);
									break;
								case "regex":
									querysetcell(qFormFields,"Parameters",".*",i);
									break;
							}
						}
						if (isdefined("form.parameters_#i#")) querysetcell(qFormFields,"Parameters",HTMLEditFormat(form["parameters_#i#"]),i);
						querysetcell(qFormFields,"Type",form["type_#i#"],i);
					}
				}
			}
		</cfscript>
		<cfquery dbtype="query" name="qFormFields">select * from qFormFields order by [position]</cfquery>
	
		<cfwddx action="cfml2wddx" input="#qFormFields#" output="form.formfields">
		
		<cfset stObj.title=form.title>
		<cfset stObj.type=form.type>
		<cfset stObj.processing=form.processing>
		<cfset stObj.successmessage=form.successmessage>
		<cfset stObj.formfields=form.formfields>
		
		<q4:contentobjectdata typename="#application.rules.ruleForm.rulePath#" stProperties="#stObj#" objectID="#stObj.objectID#">
		<cfset message = "Update successfull">
	</cfif>
	
	<cfif isDefined("message")>
		<div align="center"><strong>#message#</strong></div>
	</cfif>
				
	<!--- form --->
	<cfform action="" method="POST" class="f-wrap-2">
		<input type="hidden" name="formfields" value="#form.formfields#">
		<table width="100%" align="center" border="0" style="border:0px none">
			<input type="hidden" name="ruleID" value="#stObj.objectID#">
			<tr>
				<td align="right" style="border:0px none"><b>Title</b></td>
				<td style="border:0px none"> <input type="text" name="title" value="#stObj.title#"></td>
				<td align="right" style="border:0px none"><b>Type</b></td>
				<td style="border:0px none"><select name="type">
					<option <cfif not len(trim(stObj.type))>selected</cfif> value="">None</option>
					<cfloop collection="#application.types#" item="thistype">
						<cfif thistype neq "types"><option <cfif stObj.type eq thistype>selected</cfif> value="#thistype#">#application.types[thistype].displayname#</option></cfif>
					</cfloop>
				</td>
			</tr>
			<tr>
				<td align="right" style="border:0px none"><b>Success message</b></td>
				<td style="border:0px none"><input type="text" name="successmessage" value="#form.successmessage#"></td>
			</tr>
			<cfif len(trim(form.type))>
			<tr>
				<td align="right" style="border:0px none"><b>Processing</b></td>
				<td style="border:0px none;">
					<nj:listTemplates typename="#form.type#" prefix="process" r_qMethods="qMethods">
					<select name="processing" id="processing">
						<option value=""<cfif form.processing eq ""> selected</cfif>>None</option>
						<cfloop query="qMethods">
						<option value="#methodname#"<cfif methodname eq form.processing> selected</cfif>>#displayname#</option>
						</cfloop>
					</select>
				</td>
			</tr>
			</cfif>
		</table>
		
		<!--- Script and css for open/close layers --->
		<script language="javascript" type="text/javascript">
		<!--
			function selectField(which,total)
			{
				var i;
				for (i=1; i<=total; i++)
				{
					var thisheader = document.getElementById("fieldheader_"+i);
					var thisform = document.getElementById("fieldform_"+i);
					var thisspan = document.getElementById("fieldspan_"+i);
					
					if (i==which && thisspan.innerHTML=='+')
					{
						thisheader.style.background='##E17000'; 
						thisheader.style.border='1px solid ##E17000'; 
						thisheader.style.color='##FFFFFF';
						thisform.style.display='block';
						thisspan.innerHTML='-';
					}
					else
					{
						thisheader.style.background='white'; 
						thisheader.style.border='1px solid ##999999'; 
						thisheader.style.color='##999999';
						thisform.style.display='none';
						thisspan.innerHTML='+';
					}
				}
			}
			function moveUp(tableid,irowindex)
			{
				if (irowindex > 0)
				{
					var thistable=document.getElementById(tableid);
					
					var temphtml=thistable.rows[irowindex-1].cells[1].innerHTML;
					thistable.rows[irowindex-1].cells[1].innerHTML=thistable.rows[irowindex].cells[1].innerHTML;
					thistable.rows[irowindex].cells[1].innerHTML=temphtml;
					
					thistable.rows[irowindex-1].cells[1].getElementsByTagName('input')[0].value=irowindex;
					thistable.rows[irowindex].cells[1].getElementsByTagName('input')[0].value=irowindex+1;
				}
			}
			function moveDown(tableid,irowindex)
			{
				var thistable=document.getElementById(tableid);
				if (irowindex < thistable.rows.length-1)
				{
					var thistable=document.getElementById(tableid);
					
					var temphtml=thistable.rows[irowindex+1].cells[1].innerHTML;
					thistable.rows[irowindex+1].cells[1].innerHTML=thistable.rows[irowindex].cells[1].innerHTML;
					thistable.rows[irowindex].cells[1].innerHTML=temphtml;
					
					thistable.rows[irowindex+1].cells[1].getElementsByTagName('input')[0].value=irowindex+2;
					thistable.rows[irowindex].cells[1].getElementsByTagName('input')[0].value=irowindex+1;
				}
			}
		-->
		</script>
		<style type="text/css">
		<!--
			a.fieldformheader {
				font-weight:bold; 
				text-align:left; 
				background-color:white; 
				border: 1px solid ##999999; 
				margin-bottom: 3px;
				text-decoration: none;
				color: ##999999;
				display: block;
			}
			a.fieldformheader:hover {
				font-weight:bold; 
				text-align:left; 
				background-color:white; 
				border: 1px solid ##999999; 
				margin-bottom: 3px;
				text-decoration: none;
				color: ##000000;
				display: block;
			}
			span.fieldformheader {
				font-weight:bold; 
				text-align:left; 
				background-color:transparent; 
				border-style: none; 
				margin: 0px;
				text-decoration: none;
				font-family:monospace;
				margin-left: 5px;
			}
			span.fieldformheader:hover {
				font-weight:bold; 
				text-align:left; 
				background-color:transparent; 
				border: 0px none ##999999; 
				margin: 0px;
				text-decoration: none;
				font-family:monospace;
				margin-left: 5px;
			}
			div.fieldformsection {
				border: 0px none ##999999; 
				margin-bottom: 3px;
				background-color:transparent; 
				display:none;
			}
		-->
		</style>
		
		<cfoutput>
		<Strong>Editable fields</Strong>
		<cfwddx action="wddx2cfml" input="#form.formfields#" output="qFormFields">
		<table border="0" id="fieldstable" width="100%" style="border: 0px none;">
		<cfloop query="qFormFields">
			<cfset sttmp=application.types[form.type].stprops[property]>
			
			<tr style="margin:0px;padding:0px;border:0px none" id="propertysection_#currentrow#">
				<td style="border: 0px none; width:45px;" valign="top">
					<input name="up_#currentrow#" id="up_#currentrow#" style="background:##F1F1F1 url('/farcry/css/images/sprites_icons.gif') no-repeat 0 -758px;border:0px none;margin:0px;padding:0px;width:21px;height:22px;" type="button" value="" onclick='moveUp("fieldstable",this.parentNode.parentNode.rowIndex);' onmouseover="this.style.backgroundPosition='0 -908px'" onmouseout="this.style.backgroundPosition='0 -758px'" alt="Move Up">
					<input name="down_#currentrow#" id="down_#currentrow#" style="background:##F1F1F1 url('/farcry/css/images/sprites_icons.gif') no-repeat 0 -708px;border:0px none;margin:0px;padding:0px;width:21px;height:22px;" type="button" value="" onclick='moveDown("fieldstable",this.parentNode.parentNode.rowIndex);' onmouseover="this.style.backgroundPosition='0 -858px'" onmouseout="this.style.backgroundPosition='0 -708px'" alt="Move Down">
				</td>
				<td style="border: 0px none;">
					<input type="hidden" id="position" name="position_#currentrow#" value="#position#">
					<div style="margin:0px;padding:0px;border:0px none;"><a href="javascript:void(0)" onclick="selectField(#currentrow#,#qFormFields.recordcount#)" class="fieldformheader" id="fieldheader_#currentrow#"><span id="fieldspan_#currentrow#" class="fieldformheader">+</span> #property#</a></div>
					<div id="fieldform_#currentrow#" class="fieldformsection">
						<table cellpadding="0" cellspacing="0" style="margin:0px;padding:0px;border:0px none">
							<tr><td style="border:0px none"><input style="" type="checkbox" id="display_#currentrow#" name="display_#currentrow#" value="1" <cfif display>checked="true"</cfif>>Display</td><td style="border:0px none"><cfif required eq 2><strong>Required</strong><input type="hidden" name="required_#currentrow#" value="2"><cfelse><input style="" type="checkbox" id="required_#currentrow#" name="required_#currentrow#" value="1" <cfif required>checked="true"</cfif>>Required</cfif></td></tr>
							<tr><td style="border:0px none" colspan="2"><label for="separated_#currentrow#"><input style="clear:none;" type="checkbox" id="separated_#currentrow#" name="separated_#currentrow#" value="1" <cfif separated>checked="true"</cfif>>Seperate from previous</label></td></tr>
							<tr><td style="border:0px none">Caption:</td><td style="border:0px none"><input id="caption_#currentrow#" class="textfield wide" type="text" name="caption_#currentrow#" value="#caption#">&nbsp;&nbsp;&nbsp;   </td></tr>
							<tr><td style="border:0px none">Type:</td><td style="border:0px none">
								<select name="type_#currentrow#">
									<cfswitch expression="#sttmp.metadata.type#">
										<cfcase value="nstring,string">
											<option value="creditcard" <cfif type eq "creditcard">selected</cfif>>Credit Card</option>
											<option value="email" <cfif type eq "email">selected</cfif>>Email</option>
											<option value="text" <cfif type eq "text">selected</cfif>>Plain text</option>
											<option value="phonenumber" <cfif type eq "phonenumber">selected</cfif>>Phone Number</option>
											<option value="postcode" <cfif type eq "postcode">selected</cfif>>Postcode</option>
											<option value="selection" <cfif type eq "selection">selected</cfif>>Selection</option>
											<option value="money" <cfif type eq "money">selected</cfif>>Money</option>
											<option value="regex" <cfif type eq "regex">selected</cfif>>Regular Expression</option>
										</cfcase>
										<cfcase value="longchar">
											<option value="textarea" <cfif type eq "textarea">selected</cfif>>Text Area</option>
										</cfcase>
										<cfcase value="date">
											<option value="futuredate" <cfif type eq "futuredate">selected</cfif>>Future Date</option>
											<option value="manualdate" <cfif type eq "manualdate">selected</cfif>>Manual Date and Time</option>
											<option value="pastdate" <cfif type eq "pastdate">selected</cfif>>Past Date</option>
											<option value="time" <cfif type eq "time">selected</cfif>>Time</option>
										</cfcase>
										<cfcase value="numeric">
											<option value="decimal" <cfif type eq "decimal">selected</cfif>>Decimal Number</option>
											<option value="integer" <cfif type eq "integer">selected</cfif>>Integer</option>
											<option value="money" <cfif type eq "money">selected</cfif>>Money</option>
										</cfcase>
										<cfcase value="boolean">
											<option value="checkbox" <cfif type eq "checkbox">selected</cfif>>Checkbox</option>
											<option value="truefalse" <cfif type eq "truefalse">selected</cfif>>True / False</option>
											<option value="yesno" <cfif type eq "yesno">selected</cfif>>Yes / No</option>
										</cfcase>
										<cfcase value="array,uuid">
											<option value="ignore" <cfif type eq "ignore">selected</cfif>>Ignore field</option>
										</cfcase>
									</cfswitch>
								</select>
							</td></tr>
							<cfif listcontains("selection,text,regex",type)>
								<tr><td style="border:0px none" colspan='4'>
									<cfswitch expression="#type#">
										<cfcase value="type">Length (up to 255 characters)</cfcase>
										<cfcase value="selection">List items (separated by commas)</cfcase>
										<cfcase value="regex">Regular expression to match input to</cfcase>
									</cfswitch></td><td style="border:0px none"><input type="text" class="textfield wide" name="parameters_#currentrow#" id="parameters_#currentrow#" value="#parameters#" maxlength="255"></td></tr>
							</cfif>
							<tr><td style="border:0px none">Default:</td><td style="border:0px none">
								<cfswitch expression="#type#">
									<cfcase value="nstring,string">
										<INPUT type="text" class="textfield wide" name="default_#currentrow#" id="default_#currentrow#" value="#default#" maxlength="255" />
									</cfcase>
							
									<cfcase value="longchar">
										<TEXTAREA name="default_#currentrow#" id="default_#currentrow#" cols="40" rows="4" wrap="VIRTUAL" maxlength="2000" class="wide">#default#</TEXTAREA>
									</cfcase>
									
									<cfdefaultcase>
										<INPUT type="text" class="textfield wide" name="default_#currentrow#" id="default_#currentrow#" value="#default#" maxlength="255" />
									</cfdefaultcase>
								</cfswitch>
							</td></tr>
						</table>
					</div>
				</td>
			</tr>
		</cfloop>
		</table>
		</cfoutput>

		<div align="center"><input class="normalbttnstyle" type="submit" value="#application.adminBundle[session.dmProfile.locale].go#" name="submit"></div>
	</cfform>
</cffunction>

<cffunction name="execute" hint="Displays the form on the page" output="false" returntype="void">
	<cfargument name="objectID" required="Yes" type="uuid" default="">
	<cfargument name="dsn" required="false" type="string" default="#application.dsn#">
		<cfset var stObj = getData(arguments.objectid)>
		<cfset var types = "">
		
		<cfwddx action="wddx2cfml" input="#stObj.formfields#" output="qFormFields">
		
		<cfsavecontent variable="blurb">
			<cfoutput>
				<cfif len(trim(stObj.title))>
					<h2>#stObj.title#</h2>
				</cfif>
			</cfoutput>
		</cfsavecontent>
		
		<cfif len(trim(blurb))>
			<cfset tmp = arrayAppend(request.aInvocations,blurb)>
		</cfif>
		
		<cfif isdefined("form.submit")>
			<cfset errors="">
			<cfset success="">
			<cfset tmpObj=structnew()>
			<cfloop query="qFormFields">
				<cfset validatem = validatefield(whichfield=currentrow,required=required,type=type,caption=caption,parameters=parameters)>
				<cfif len(trim(validatem)) and type neq "ignore">
					<cfset errors = "#errors##validatem#">
				<cfelse>
					<cfif type is "money">
						<cfset tmpObj[property]=lsparsecurrency(form["field_#currentrow#"])>
					<cfelse>
						<cfset tmpObj[property]=form["field_#currentrow#"]>
						<cfif lcase(property) eq "title"><cfset tmpObj.label=form["field_#currentrow#"]></cfif>
					</cfif>
				</cfif>
			</cfloop>
			<cfif not len(trim(errors)) and len(trim(stObj.type))>
				<cfif isdefined("stObj.successmessage") and len(trim(stObj.successmessage))><cfset success="<li>#stObj.successmessage#</li>"></cfif>
				<cfset thistype=application.types[stObj.type]>
				<cfinvoke component="#thistype.typepath#" method="createData" returnvariable="newinstance">
					<cfinvokeargument name="stProperties" value="#tmpObj#">
					<cfinvokeargument name="user" value="anonymous">
					<cfinvokeargument name="auditNote" value="Created by ruleForm">
				</cfinvoke>
				<cfif isdefined("stObj.processing") and len(trim(stObj.processing))>
					<cfset oTemp=CreateObject("component",thistype.typepath)>
					<cfsavecontent variable="blurb">
						<cfset oTemp.getDisplay(objectid=newinstance.objectid,template=stObj.processing)>
					</cfsavecontent>
					<cfif len(trim(blurb))>
						<cfset tmp = arrayAppend(request.aInvocations,blurb)>
					</cfif>
				</cfif>
				<cfif isdefined("success") and len(trim(success))>
					<cfset blurb="<ul class='success'>#success#</ul>">
					<cfset tmp = arrayAppend(request.aInvocations,blurb)>
				</cfif>
			</cfif>
		</cfif>
		
		<cfsavecontent variable="blurb">
			<cfoutput>
				<cfif isdefined("errors") and len(trim(errors))><ul class="error">#errors#</ul></cfif>
				
				<cfif len(trim(stObj.type))>
					<form action='' method='post' name='ruleform' id='ruleform'>
					<table border='0'>
					
					<cfloop query="qFormFields">
						<cfif display and type neq "ignore">
							<cfif separated><tr><td>&nbsp; </td></tr></cfif>
							<tr>
								<td>#caption#<cfif required> <span class="required">*</span></cfif></td>
								<td>#forminput(currentrow,qFormFields.type,parameters,default)#</td>
							</tr>
						<cfelse>
							<input type='hidden' name='field_#currentrow#' value='#default#'>
						</cfif>
					</cfloop>
					<tr><td colspan='2'><input type='submit' name='submit' value='Submit' /></td></tr>
					</table>
					</form>
				</cfif>
			</cfoutput>
		</cfsavecontent>
		
		<cfif len(trim(blurb))>
			<cfset tmp = arrayAppend(request.aInvocations,blurb)>
		</cfif>
	</cffunction>

</cfcomponent>

